/*
 * MSP430 LCR meter
 * Free for non-commercial use.
 * 
 * Contact: xyphro@gmail.com
 */
 
#include "key.h"
#include <intrinsics.h>
#include <io430.h>
#include "sys.h"

/*
  Access Keys on P1.0 and P1.1
*/

void key_init(void)
{
  P1DIR &= ~ ((1 << 0) | (1 << 1));                                             /* set port directions to input */
  P1REN |=   ((1 << 0) | (1 << 1));                                             /* enable resistors */
  P1OUT |=   ((1 << 0) | (1 << 1));                                             /* pullup resistors */
}


uint8_t get_key(void)
{
  uint8_t result;
  
  result = 0;
  if ( !(P1IN & (1 << 0)) )
  { /* Key on P1.0 is down */
    result |= KEY_S1;
  }
  if ( !(P1IN & (1 << 0)) )
  { /* Key on P1.0 is down */
    result |= KEY_S2;
  }
  return result;
}

/* wait until a key was pressed*/
uint8_t wait_key(void)
{
  uint8_t result;
  uint32_t cnt;
  
  
  /* wait for new key down event */
  do
  {
    result = get_key();
  } while ( result == 0 );
  
  cnt = 0;
  /* wait until key was released */
  while (get_key() != 0)
  {
    __delay_cycles(1200); /* wait 10 ms */
    cnt++;
    if (cnt > 20000)
    { /* goto LPM! */
      sys_standby();
      result = 0;
    }
  }
  
  
  return result;
}

