/*
 * MSP430 LCR meter
 * Free for non-commercial use.
 * 
 * Contact: xyphro@gmail.com
 */

#include "io430.h"
#include "pga.h"
#include <intrinsics.h>                                                         /* needed for __delay_cycles macro */


const uint16_t pga_gainvalues[PGA_GAINCOUNT] = {1, 2, 5, 10, 20, 50, 100, 200};


/* write 16 bit via software SPI bus to PGA11x (MSB first) */
/* For bitpattern documentation see pga112.pdf, page 27, table 3 */
void s_pga11x_write(uint16_t value)
{
  uint8_t cnt;
  P6OUT &= ~(1 << 3);                                                           /* CS = 0 */
  for (cnt=16; cnt > 0; cnt--)
  {
    if (value & 0x8000)                                                         /* set DIO according to current bit value */
      P6OUT |=  (1 <<4);
    else
      P6OUT &= ~(1 <<4);
    P6OUT |=  (1 << 5);                                                         /* SCLK = 1 */
    __delay_cycles(5);
    P6OUT &= ~(1 << 5);                                                         /* SCLK = 0 */
    value <<= 1;
  }
  __delay_cycles(5);
  P6OUT |= (1 << 3);                                                            /* CS = 1 */
}

/* channel: 
   0 = CH0
   1 = CH1
  12 = CAL1 (GND)
  13 = CAL2 (0.9V*VCAL)
  14 = CAL3 (0.1V*VCAL)
  15 = CAL4 (VREF)
 */
void pga11x_set(uint8_t gain, uint8_t channel)
{
  gain    &= 0x0f;
  channel &= 0x0f;
  s_pga11x_write( 0x2a00 | (gain << 4) | channel );
}

void pga_init(void)
{
  P6DIR |= (1 << 5) | (1 << 4) | (1 << 3);                                      /* initialize IO lines */
  P6OUT |= (1 << 3);                                                            /* /CS = high */
  P6OUT &= ~((1 << 5) | (1 << 4));                                              /* SCLK & DIO = low */
}



