/*
 * MSP430 LCR meter
 * Free for non-commercial use.
 * 
 * Contact: xyphro@gmail.com
 */
#ifndef _SINETABS_H
#define _SINETABS_H

#include <stdint.h>

/* 
  Theese tables contain signal arrays for sinusodial signal generation. They are
  "played back" with a 1.2 MHz sample rate using DMA and DAC12 channel 0.

  The root causes for choosing 1.2 MHz as sample rate are:
  - A high sample rate allows efficient filtering of the harmonics generated by the
    sampling rate, especially for high frequencies due to larger steps
  - Why 1.2 Mhz and not 1 MHz? When a single sinusodial cycle uses a multiple of 4 
    samples it is possible to use the sample table also as DFT twiddle table, because
    the needed phase shift of 90 degrees can be easily realized by shifting of 1/4 
    of the amplitude For 1 MHz and 10 kHz this a single sine would have a period 
    of 100 samples, which is not divideable by 4, so I have choosen 12 MHz where
    this value is 120 samples...

  This combined table approach saves flash memory and also increases speed for 
  reflashing which is very important for my motivation during initial programming :-) 

  DFT Twiddles... take the next smaller sine table 
  (because adc conversion rate is effectively 1/10 of DAC sample rate):
  * Imaginery twiddles = directly the sine table
  * real twiddles = sine table, shifted "left" by "sample count>>2"

  construction of the sample arrays:
  - raw sample data (signed 12 Bit right adjusted format, 95% of full amplitude)  
 */

// 100 kHz - length 12
extern const int16_t Sin_tab_100kHz[12];

// 10 kHz - length 120
extern const int16_t Sin_tab_10kHz[120];

// 1 kHz - length 1200
extern const int16_t Sin_tab_1kHz[1200];

// 100 Hz - length 12000
extern const int16_t Sin_tab_100Hz[12000];

// 1000 Hz - length 1200 used for calibration, it has a lower amplitude to be able to calibrate the PGA
extern const int16_t Sin_tab_1000Hz_calib[1200];

#endif