/*
 * MSP430 LCR meter
 * Free for non-commercial use.
 * 
 * Contact: xyphro@gmail.com
 */

#include "io430.h"
#include "sys.h"
#include <intrinsics.h>                                                         /* needed for __dleay_cycles macro */
#include "dac.h"



static void s_sys_init_clock(void)
{
#if 0
  UCSCTL3 |= SELREF_2;                      // Set DCO FLL reference = REFO
  UCSCTL4 |= SELA_2;                        // Set ACLK = REFO
  __bis_SR_register(SCG0);                  // Disable the FLL control loop
  UCSCTL0 = 0x0000;                         // Set lowest possible DCOx, MODx
  UCSCTL1 = DCORSEL_5;                      // Select DCO range 24MHz operation
  UCSCTL2 = FLLD_1 + 374;                   // Set DCO Multiplier for 12MHz
                                            // (N + 1) * FLLRef = Fdco
                                            // (374 + 1) * 32768 = 12MHz
                                            // Set FLL Div = fDCOCLK/2
  __bic_SR_register(SCG0);                  // Enable the FLL control loop


  // Worst-case settling time for the DCO when the DCO range bits have been
  // changed is n x 32 x 32 x f_MCLK / f_FLL_reference. See UCS chapter in 5xx
  // UG for optimization.
  // 32 x 32 x 12 MHz / 32,768 Hz = 375000 = MCLK cycles for DCO to settle
  __delay_cycles(375000);
  
  // Loop until XT1,XT2 & DCO fault flag is cleared
  do
  {
    UCSCTL7 &= ~(XT2OFFG + XT1LFOFFG + DCOFFG);
                                            // Clear XT2,XT1,DCO fault flags
    SFRIFG1 &= ~OFIFG;                      // Clear fault flags
  }
  while (SFRIFG1&OFIFG);                   // Test oscillator fault flag
  
#else
  
  P7SEL |= BIT2+BIT3;                       // Port select XT2
  
  UCSCTL6 &= ~XT2OFF;                       // Enable XT2 
  UCSCTL3 |= SELREF_2;                      // FLLref = REFO
                                            // Since LFXT1 is not used,
                                            // sourcing FLL with LFXT1 can cause
                                            // XT1OFFG flag to set
  UCSCTL4 |= SELA_2;                        // ACLK=REFO,SMCLK=DCO,MCLK=DCO

  // Loop until XT1,XT2 & DCO stabilizes - in this case loop until XT2 settles
  do
  {
    UCSCTL7 &= ~(XT2OFFG + XT1LFOFFG + DCOFFG);
                                            // Clear XT2,XT1,DCO fault flags
    SFRIFG1 &= ~OFIFG;                      // Clear fault flags
  }while (SFRIFG1&OFIFG);                   // Test oscillator fault flag

  UCSCTL6 &= ~XT2DRIVE0;                    // Decrease XT2 Drive according to
                                            // expected frequency
  UCSCTL4 |= SELS_5 + SELM_5;               // SMCLK=MCLK=XT2
#endif
}





void sys_init(void)
{
  PMMCTL0_H = PMMPW_H;
  PM5CTL0 = 0x00; /* reset LPM5LOCK bit */

  P4OUT = 0;
  P4DIR = 0xff;
  P3OUT = 0;
  P3DIR = 0xff;
  P2OUT = 0;
  P2DIR = 0xff;
  P1OUT = 0x03;
  P1DIR = 0xfc;
  P9OUT = 0;
  P9DIR = 0xff;
  P5DIR |= 0xfe;
  P5OUT = 0;
  P7DIR = 0xff;
  P7OUT = 0;
  
  /* enable port u output (used to switch power on the system) */
  LDOKEYPID = 0x9628;                                                           /* unlock LDO module */
  PUCTL = (1 << 5);                                                             /* enable, PU.0 and PU.1 are low*/
  LDOKEYPID = 0x0000;                                                           /* lock LDO module */

  /* initialize clock to 12 MHz */
  s_sys_init_clock();
  
}



void sys_extpower_switch(bool pwrOn)
{
  LDOKEYPID = 0x9628;                                                           /* unlock LDO module */
  if (pwrOn)
    PUCTL |=   (1 << 1) | (1 << 0);
  else
  {
    PUCTL &= ~(1 << 5);
    PUCTL &= ~((1 << 1) | (1 << 0));
    //LDOPWRCTL = 0x0000;
  }
  LDOKEYPID = 0x0000;                                                           /* lock LDO module */
}

#pragma vector = PORT1_VECTOR
__interrupt void POWERUP_ISR(void)
{
  __delay_cycles(12000ul*5ul); /* wait 500 ms */
  __bic_SR_register_on_exit(LPM4_bits);
  WDTCTL = WDTPW | WDTIS__64 | WDTCNTCL_L;
/*  while(1)
  {
    __no_operation();
    __no_operation();
    __no_operation();
  }*/ 
}

void sys_standby(void)
{
  P4OUT = 0;
  P4DIR = 0xff;
  P3OUT = 0;
  P3DIR = 0xff;
  P2OUT = 0;
  P2DIR = 0xff;
  P1OUT = 0x03;
  P1DIR = 0xfc;
  P9OUT = 0;
  P9DIR = 0xff;
  P5DIR |= 0xfe;
  P5OUT = 0;
  P7DIR = 0xff;
  P7OUT = 0;
  
  /* power down lcd */
  P8OUT = 0x00;
  P6OUT = 0x00;
  sys_extpower_switch(false);
  __delay_cycles(12000000ul); /* wait 1 s*/
  
  UCA1CTL1 = UCSWRST;
  dac_stop_wave();
  UCB1CTL0 = 0;
  UCB1CTL1 = UCSWRST;
  DAC12_0CTL0 = 0;
  DAC12_0CTL0 = 0;
  
  P1IES = 0x01;
  P1IE  = 0x01; /* enable P1.0 interrupt */
  P1IFG = 0x00; /* clear interrupts */
  
  UCSCTL8 = 0x00; /* disable clock requests*/
  UCSCTL4 = (1 << 8) | (1 << 4) | (1 << 0); // all clock sources = VLO
  UCSCTL6 = 0xC1CD; /* disable XT2 Oscillator */
    
  PMMCTL0_H = PMMPW_H;
  PMMCTL0_L |= PMMREGOFF;
  __bis_SR_register(LPM4_bits + GIE);
  __no_operation();
  while(1);
}
